@@ -1,35 +1,21 @@
-use crate::apiv1::conn_pool::{AUDIENCE, SPANNER};
-
-use google_cloud_gax::call_option::{Backoff, BackoffRetrySettings, BackoffRetryer};
-use google_cloud_gax::invoke::invoke_reuse;
-use google_cloud_gax::util::create_request;
 use google_cloud_googleapis::iam::v1::{
-    GetIamPolicyRequest, Policy, SetIamPolicyRequest, TestIamPermissionsRequest,
-    TestIamPermissionsResponse,
+    GetIamPolicyRequest, Policy, SetIamPolicyRequest, TestIamPermissionsRequest, TestIamPermissionsResponse,
 };
 
 use google_cloud_googleapis::spanner::admin::instance::v1::instance_admin_client::InstanceAdminClient as InternalInstanceAdminClient;
-use google_cloud_googleapis::{Code, Status};
 
-use crate::admin::SCOPES;
+use crate::admin::{default_internal_client, default_retry_setting};
+use google_cloud_gax::cancel::CancellationToken;
+use google_cloud_gax::conn::{Channel, Error};
+use google_cloud_gax::create_request;
+use google_cloud_gax::grpc::{Response, Status};
+use google_cloud_gax::retry::{invoke, RetrySetting};
 use google_cloud_googleapis::spanner::admin::instance::v1::{
-    CreateInstanceRequest, DeleteInstanceRequest, GetInstanceConfigRequest, GetInstanceRequest,
-    Instance, InstanceConfig, ListInstanceConfigsRequest, ListInstancesRequest,
-    UpdateInstanceRequest,
+    CreateInstanceRequest, DeleteInstanceRequest, GetInstanceConfigRequest, GetInstanceRequest, Instance,
+    InstanceConfig, ListInstanceConfigsRequest, ListInstancesRequest, UpdateInstanceRequest,
 };
-use google_cloud_grpc::conn::{Channel, ConnectionManager, Error};
 use google_cloud_longrunning::autogen::operations_client::OperationsClient;
 use google_cloud_longrunning::longrunning::Operation;
-use tonic::Response;
-
-fn default_setting() -> BackoffRetrySettings {
-    BackoffRetrySettings {
-        retryer: BackoffRetryer {
-            backoff: Backoff::default(),
-            codes: vec![Code::Unavailable, Code::Unknown, Code::DeadlineExceeded],
-        },
-    }
-}
 
 #[derive(Clone)]
 pub struct InstanceAdminClient {
@@ -43,53 +29,32 @@ impl InstanceAdminClient {
     }
 
     pub async fn default() -> Result<Self, Error> {
-        let emulator_host = match std::env::var("SPANNER_EMULATOR_HOST") {
-            Ok(s) => Some(s),
-            Err(_) => None,
-        };
-        let conn_pool =
-            ConnectionManager::new(1, SPANNER, AUDIENCE, Some(&SCOPES), emulator_host).await?;
-        let conn = conn_pool.conn();
-        let lro_client = OperationsClient::new(conn).await?;
-        let conn = conn_pool.conn();
-        Ok(Self::new(
-            InternalInstanceAdminClient::new(conn),
-            lro_client,
-        ))
-    }
-
-    /// merge call setting
-    fn get_call_setting(call_setting: Option<BackoffRetrySettings>) -> BackoffRetrySettings {
-        match call_setting {
-            Some(s) => s,
-            None => default_setting(),
-        }
+        let (conn, lro_client) = default_internal_client().await?;
+        Ok(Self::new(InternalInstanceAdminClient::new(conn), lro_client))
     }
 
     /// list_instance_configs lists the supported instance configurations for a given project.
     pub async fn list_instance_configs(
-        &mut self,
+        &self,
         mut req: ListInstanceConfigsRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Vec<InstanceConfig>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let parent = &req.parent;
         let mut all = vec![];
         //eager loading
         loop {
-            let response = invoke_reuse(
-                |client| async {
-                    let request = create_request(format!("parent={}", parent), req.clone());
-                    client
-                        .list_instance_configs(request)
-                        .await
-                        .map_err(|e| (Status::from(e), client))
-                        .map(|d| d.into_inner())
-                },
-                &mut self.inner,
-                &mut setting,
-            )
-            .await?;
+            let action = || async {
+                let request = create_request(format!("parent={}", parent), req.clone());
+                self.inner
+                    .clone()
+                    .list_instance_configs(request)
+                    .await
+                    .map_err(|e| e.into())
+                    .map(|d| d.into_inner())
+            };
+            let response = invoke(cancel.clone(), retry.clone(), action).await?;
             all.extend(response.instance_configs.into_iter());
             if response.next_page_token.is_empty() {
                 return Ok(all);
@@ -100,50 +65,47 @@ impl InstanceAdminClient {
 
     /// get_instance_config gets information about a particular instance configuration.
     pub async fn get_instance_config(
-        &mut self,
+        &self,
         req: GetInstanceConfigRequest,
-        opt: Option<BackoffRetrySettings>,
-    ) -> Result<Response<InstanceConfig>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
+    ) -> Result<InstanceConfig, Status> {
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let name = &req.name;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("name={}", name), req.clone());
-                client
-                    .get_instance_config(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await;
+        let action = || async {
+            let request = create_request(format!("name={}", name), req.clone());
+            self.inner
+                .clone()
+                .get_instance_config(request)
+                .await
+                .map_err(|e| e.into())
+                .map(|d| d.into_inner())
+        };
+        invoke(cancel, retry, action).await
     }
 
     /// list_instances lists all instances in the given project.
     pub async fn list_instances(
-        &mut self,
+        &self,
         mut req: ListInstancesRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Vec<Instance>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let parent = &req.parent;
         let mut all = vec![];
         //eager loading
         loop {
-            let response = invoke_reuse(
-                |client| async {
-                    let request = create_request(format!("parent={}", parent), req.clone());
-                    client
-                        .list_instances(request)
-                        .await
-                        .map_err(|e| (Status::from(e), client))
-                        .map(|d| d.into_inner())
-                },
-                &mut self.inner,
-                &mut setting,
-            )
-            .await?;
+            let action = || async {
+                let request = create_request(format!("parent={}", parent), req.clone());
+                self.inner
+                    .clone()
+                    .list_instances(request)
+                    .await
+                    .map_err(|e| e.into())
+                    .map(|d| d.into_inner())
+            };
+            let response = invoke(cancel.clone(), retry.clone(), action).await?;
             all.extend(response.instances.into_iter());
             if response.next_page_token.is_empty() {
                 return Ok(all);
@@ -154,24 +116,18 @@ impl InstanceAdminClient {
 
     /// gets information about a particular instance.
     pub async fn get_instance(
-        &mut self,
+        &self,
         req: GetInstanceRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Response<Instance>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let name = &req.name;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("name={}", name), req.clone());
-                client
-                    .get_instance(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await;
+        let action = || async {
+            let request = create_request(format!("name={}", name), req.clone());
+            self.inner.clone().get_instance(request).await.map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action).await
     }
 
     /// create_instance creates an instance and begins preparing it to begin serving. The
@@ -216,23 +172,18 @@ impl InstanceAdminClient {
     pub async fn create_instance(
         &mut self,
         req: CreateInstanceRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Operation<Instance>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let parent = &req.parent;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("parent={}", parent), req.clone());
-                client
-                    .create_instance(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await
-        .map(|d| Operation::new(self.lro_client.clone(), d.into_inner()));
+        let action = || async {
+            let request = create_request(format!("parent={}", parent), req.clone());
+            self.inner.clone().create_instance(request).await.map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action)
+            .await
+            .map(|d| Operation::new(self.lro_client.clone(), d.into_inner()))
     }
 
     /// update_instance updates an instance, and begins allocating or releasing resources
@@ -280,26 +231,20 @@ impl InstanceAdminClient {
     /// Authorization requires spanner.instances.update permission on
     /// resource [name][google.spanner.admin.instance.v1.Instance.name (at http://google.spanner.admin.instance.v1.Instance.name)].
     pub async fn update_instance(
-        &mut self,
+        &self,
         req: UpdateInstanceRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Operation<Instance>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let instance_name = &req.instance.as_ref().unwrap().name;
-        return invoke_reuse(
-            |client| async {
-                let request =
-                    create_request(format!("instance.name={}", instance_name), req.clone());
-                client
-                    .update_instance(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await
-        .map(|d| Operation::new(self.lro_client.clone(), d.into_inner()));
+        let action = || async {
+            let request = create_request(format!("instance.name={}", instance_name), req.clone());
+            self.inner.clone().update_instance(request).await.map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action)
+            .await
+            .map(|d| Operation::new(self.lro_client.clone(), d.into_inner()))
     }
 
     /// DeleteInstance deletes an instance.
@@ -314,24 +259,18 @@ impl InstanceAdminClient {
     ///   irrevocably disappear from the API. All data in the databases
     ///   is permanently deleted.
     pub async fn delete_instance(
-        &mut self,
+        &self,
         req: DeleteInstanceRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Response<()>, Status> {
-        let mut setting = Self::get_call_setting(opt);
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
         let name = &req.name;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("parent={}", name), req.clone());
-                client
-                    .delete_instance(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await;
+        let action = || async {
+            let request = create_request(format!("name={}", name), req.clone());
+            self.inner.clone().delete_instance(request).await.map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action).await
     }
 
     /// set_iam_policy sets the access control policy on an instance resource. Replaces any
@@ -339,24 +278,18 @@ impl InstanceAdminClient {
     ///
     /// Authorization requires spanner.instances.setIamPolicy on resource.
     pub async fn set_iam_policy(
-        &mut self,
+        &self,
         req: SetIamPolicyRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Response<Policy>, Status> {
-        let mut setting = Self::get_call_setting(opt);
         let resource = &req.resource;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("resource={}", resource), req.clone());
-                client
-                    .set_iam_policy(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await;
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
+        let action = || async {
+            let request = create_request(format!("resource={}", resource), req.clone());
+            self.inner.clone().set_iam_policy(request).await.map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action).await
     }
 
     /// get_iam_policy sets the access control policy on an instance resource. Replaces any
@@ -364,24 +297,18 @@ impl InstanceAdminClient {
     ///
     /// Authorization requires spanner.instances.setIamPolicy on resource.
     pub async fn get_iam_policy(
-        &mut self,
+        &self,
         req: GetIamPolicyRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Response<Policy>, Status> {
-        let mut setting = Self::get_call_setting(opt);
         let resource = &req.resource;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("resource={}", resource), req.clone());
-                client
-                    .get_iam_policy(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await;
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
+        let action = || async {
+            let request = create_request(format!("resource={}", resource), req.clone());
+            self.inner.clone().get_iam_policy(request).await.map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action).await
     }
 
     /// test_iam_permissions returns permissions that the caller has on the specified instance resource.
@@ -391,23 +318,21 @@ impl InstanceAdminClient {
     /// permission on the containing Google Cloud Project. Otherwise returns an
     /// empty set of permissions.
     pub async fn test_iam_permissions(
-        &mut self,
+        &self,
         req: TestIamPermissionsRequest,
-        opt: Option<BackoffRetrySettings>,
+        cancel: Option<CancellationToken>,
+        retry: Option<RetrySetting>,
     ) -> Result<Response<TestIamPermissionsResponse>, Status> {
-        let mut setting = Self::get_call_setting(opt);
         let resource = &req.resource;
-        return invoke_reuse(
-            |client| async {
-                let request = create_request(format!("resource={}", resource), req.clone());
-                client
-                    .test_iam_permissions(request)
-                    .await
-                    .map_err(|e| (e.into(), client))
-            },
-            &mut self.inner,
-            &mut setting,
-        )
-        .await;
+        let retry = Some(retry.unwrap_or(default_retry_setting()));
+        let action = || async {
+            let request = create_request(format!("resource={}", resource), req.clone());
+            self.inner
+                .clone()
+                .test_iam_permissions(request)
+                .await
+                .map_err(|e| e.into())
+        };
+        invoke(cancel, retry, action).await
     }
 }
