@@ -1,5 +1,5 @@
+use google_cloud_gax::conn::{ConnectionManager as GRPCConnectionManager, Error};
 use google_cloud_googleapis::spanner::v1::spanner_client::SpannerClient;
-use google_cloud_grpc::conn::{ConnectionManager as GRPCConnectionManager, Error};
 
 use crate::apiv1::spanner_client::Client;
 
@@ -17,14 +17,7 @@ pub struct ConnectionManager {
 impl ConnectionManager {
     pub async fn new(pool_size: usize, emulator_host: Option<String>) -> Result<Self, Error> {
         Ok(ConnectionManager {
-            inner: GRPCConnectionManager::new(
-                pool_size,
-                SPANNER,
-                AUDIENCE,
-                Some(&SCOPES),
-                emulator_host,
-            )
-            .await?,
+            inner: GRPCConnectionManager::new(pool_size, SPANNER, AUDIENCE, Some(&SCOPES), emulator_host).await?,
         })
     }
 
